﻿using System;
using System.Collections.Generic;
using System.Text;
using System.Windows.Forms;

using Autodesk.Revit.DB;
using Autodesk.Revit.UI;
using Autodesk.Revit.ApplicationServices;
using Autodesk.Revit.Attributes;
using Autodesk.Revit.UI.Selection;


[TransactionAttribute(Autodesk.Revit.Attributes.TransactionMode.Manual)]
public class FindIntersectWallsByElement : IExternalCommand
{
  public Result Execute(ExternalCommandData commandData, ref string messages, ElementSet elements)
  {

    UIApplication app = commandData.Application;
    Document doc = app.ActiveUIDocument.Document;
    Transaction trans = new Transaction(doc, "ExComm");
    trans.Start();

    //pick the column
    Selection sel = app.ActiveUIDocument.Selection;
    Reference ref1 = sel.PickObject(ObjectType.Element, "Please pick a column");
    Element column = doc.GetElement(ref1);

    FilteredElementCollector collector = new FilteredElementCollector(doc);
    ElementIntersectsElementFilter elementFilter = new ElementIntersectsElementFilter(column, false);
    collector.WherePasses(elementFilter);

    List<ElementId> excludes = new List<ElementId>();
    excludes.Add(column.Id);
    collector.Excluding(excludes);

    sel.Elements.Clear();

    //Add these interseting element to the selection
    foreach (Element elem in collector)
    {
      sel.Elements.Add(elem);
    }

    trans.Commit();
    return Result.Succeeded;
  }
}


[TransactionAttribute(Autodesk.Revit.Attributes.TransactionMode.Manual)]
public class FindIntersectWallsByGeometry : IExternalCommand
{
  public Result Execute(ExternalCommandData commandData, ref string messages, ElementSet elements)
  {

    UIApplication app = commandData.Application;
    Document doc = app.ActiveUIDocument.Document;
    Transaction trans = new Transaction(doc, "ExComm");
    trans.Start();

    //pick a point to draw solid
    Selection sel = app.ActiveUIDocument.Selection;
    XYZ pt = sel.PickPoint("Please pick a point to get the close walls");

    XYZ pttemp1 = sel.PickPoint(Autodesk.Revit.UI.​Selection.ObjectSnapTypes.None, "Pick leader end...");
    XYZ pttemp2 = sel.PickPoint(Autodesk.Revit.UI.Selection.ObjectSnapTypes.None, "Pick leader elbow...");

    //
    double dBoxLength = 3;

    XYZ pt1 = new XYZ(pt.X - dBoxLength / 2, pt.Y - dBoxLength / 2, pt.Z);
    XYZ pt2 = new XYZ(pt.X + dBoxLength / 2, pt.Y - dBoxLength / 2, pt.Z);
    XYZ pt3 = new XYZ(pt.X + dBoxLength / 2, pt.Y + dBoxLength / 2, pt.Z);
    XYZ pt4 = new XYZ(pt.X - dBoxLength / 2, pt.Y + dBoxLength / 2, pt.Z);

    Line lineBottom = app.Application.Create.NewLineBound(pt1, pt2);
    Line lineRight = app.Application.Create.NewLineBound(pt2, pt3);
    Line lineTop = app.Application.Create.NewLineBound(pt3, pt4);
    Line lineLeft = app.Application.Create.NewLineBound(pt4, pt1);

    CurveLoop profile = new CurveLoop();
    profile.Append(lineBottom);
    profile.Append(lineRight);
    profile.Append(lineTop);
    profile.Append(lineLeft);

    List<CurveLoop> loops = new List<CurveLoop>();
    loops.Add(profile);

    XYZ vector = new XYZ(0, 0, 1);
    Solid solid = GeometryCreationUtilities.CreateExtrusionGeometry(loops, vector, 10);



    FilteredElementCollector collector = new FilteredElementCollector(doc);
    ElementIntersectsSolidFilter solidFilter = new ElementIntersectsSolidFilter(solid);

    collector.WherePasses(solidFilter);

    sel.Elements.Clear();
    //Add these interseting element to the selection
    foreach (Element elem in collector)
    {
      sel.Elements.Add(elem);
    }

    trans.Commit();
    return Result.Succeeded;
  }
}